import { escape } from 'html-escaper';
import { compile, syntax } from './syntax';
const compiled = new Map();
export const createStyleSheet = (keys) => {
    const style = new CSSStyleSheet();
    style.replaceSync(`*{color:var(--color);}*::selection{background:var(--selection)!important}` +
        keys.map(x => `.${x}{color:var(--${x});font-style:var(--${x}-style);font-weight:var(--${x}-weight)}`).join(''));
    return style;
};
export const languages = {};
/**
 * CodeSyntax custom element
 *
 * ```js
 * import { CodeSyntaxElement } from 'code-syntax'
 * customElements.define('code-syntax', CodeSyntaxElement)
 * ```
 */
export class CodeSyntaxElement extends HTMLElement {
    static get observedAttributes() {
        return ['language', 'theme', 'html'];
    }
    #nodes = [];
    pins = {};
    constructor() {
        super();
        const root = this.attachShadow({ mode: 'open' });
        root.innerHTML = /*html*/ `<slot style="display:none"></slot><pre style="margin:0"><code></code></pre>`;
        this.pins.code = root.querySelector('code');
        const contentObserver = new MutationObserver(this.render);
        const slot = root.querySelector('slot');
        slot.addEventListener('slotchange', () => {
            this.#nodes = slot.assignedNodes();
            this.#nodes.forEach(node => contentObserver.observe(node, { characterData: true }));
            this.render();
        });
    }
    highlight = (s) => escape(s);
    render = () => {
        this.html = this.highlight(this.#nodes
            .map(node => {
            const text = node.textContent;
            return text.trim().length ? text : '';
        })
            .join(''));
    };
    set html(html) {
        this.pins.code.innerHTML = html;
    }
    set syntax(def) {
        this.setSyntaxDefinition(def);
    }
    set styleSheet(styleSheet) {
        this.shadowRoot.adoptedStyleSheets = [styleSheet];
    }
    async setSyntaxDefinition(def) {
        let { style, regexp } = compiled.get(def) ?? {};
        if (!style) {
            regexp = await compile(def);
            style = createStyleSheet([...regexp.keys]);
            compiled.set(def, { style, regexp });
        }
        this.styleSheet = style;
        this.highlight = (s) => syntax(regexp, s);
        this.render();
    }
    attributeChangedCallback(name, _, newValue) {
        if (name === 'language' && newValue && newValue in languages)
            this.setSyntaxDefinition(languages[newValue]);
        if (name === 'theme')
            this.setAttribute('code-syntax-theme', newValue);
        if (name === 'html')
            this.html = newValue ?? '';
    }
}
//# sourceMappingURL=element.js.map